/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.fleet.machine.tree;

import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.AbstractAction;
import javax.swing.Action;
import nl.cloudfarming.client.fleet.machine.windows.OverviewTopComponent;
import nl.cloudfarming.client.fleet.model.Machine;
import nl.cloudfarming.client.geoviewer.Layer;
import nl.cloudfarming.client.geoviewer.SimpleLayer;
import org.openide.actions.DeleteAction;
import org.openide.actions.PropertiesAction;
import org.openide.actions.RenameAction;
import org.openide.filesystems.FileLock;
import org.openide.filesystems.FileObject;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.nodes.Sheet;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.util.actions.SystemAction;
import org.openide.util.datatransfer.PasteType;
import org.openide.util.lookup.Lookups;
import org.openide.util.lookup.ProxyLookup;

/**
 *
 * @author Wytse Visser && Maciek Dulkiewicz
 */
public abstract class MachineNode extends AbstractNode {

    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.fleet.machine");
    protected static final String ICON = "nl/cloudfarming/client/fleet/machine/bullet.png";
    private Machine machine;

    public MachineNode(Lookup lookup) {
        super(Children.LEAF, new ProxyLookup(lookup, Lookups.fixed(createLayer(lookup))));
        setIconBaseWithExtension(ICON);
    }
    
    private static Layer createLayer(Lookup lookup) { 
        Machine machine = lookup.lookup(Machine.class);
        return new SimpleLayer(machine.getName());
    }

    @Override
    public String getName() {
        return getMachine().getName();
    }

    @Override
    public void setName(String name) {
        String oldName = getName();

        try {
            FileLock lock = getMachineFile().lock();
            getMachineFile().rename(lock, name, getMachineFile().getExt());
            lock.releaseLock();

            getMachine().setName(name);
            fireDisplayNameChange(oldName, name);
        } catch (IOException ex) {
            LOGGER.log(Level.WARNING, "Could not rename the machine", ex.getMessage());
        }
    }

    @Override
    public void destroy() throws IOException {
        getMachineFile().delete();
    }

    protected FileObject getMachineFile() {
        return (FileObject) getLookup().lookup(FileObject.class);
    }

    @Override
    public boolean canDestroy() {
        return true;
    }

    @Override
    public boolean canRename() {
        return true;
    }

    @Override
    public Action getPreferredAction() {
        return new OpenAction();
    }

    @Override
    public Action[] getActions(boolean context) {
        return new Action[]{
                    new OpenAction(),
                    SystemAction.get(DeleteAction.class),
                    SystemAction.get(RenameAction.class),
                    SystemAction.get(PropertiesAction.class)
                };
    }

    @Override
    protected Sheet createSheet() {
        return getMachine().createSheet();
    }

    @Override
    public PasteType getDropType(Transferable t, int action, int index) {
        return null;
    }

    protected final Machine getMachine() {
        if (machine == null) {
            machine = getLookup().lookup(Machine.class);
            machine.addPropertyChangeListener(new AutoSaver());
        }
        return machine;
    }

    protected MachineCategoryNode.MachineMetaInfo getMetaInfo() {
        MachineCategoryNode categoryNode = (MachineCategoryNode) getParentNode();
        return categoryNode.getMetaInfo();
    }

    protected abstract void saveMachine();

    @NbBundle.Messages({"fleet.machine.machine_node_open_action_name=Open"})
    protected class OpenAction extends AbstractAction {

        public OpenAction() {
            super(Bundle.fleet_machine_machine_node_open_action_name());
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            OverviewTopComponent overviewComponent = OverviewTopComponent.findInstance(getMachine(), getMetaInfo());
            overviewComponent.open();
            overviewComponent.requestActive();
        }
    }

    private class AutoSaver implements PropertyChangeListener {

        @Override
        public void propertyChange(PropertyChangeEvent evt) {
            saveMachine();
        }
    }
}