/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.fleet.machine.windows;

import java.awt.BorderLayout;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.ImageIcon;
import javax.swing.SwingUtilities;
import javax.swing.table.DefaultTableModel;
import nl.cloudfarming.client.fleet.machine.tree.MachineCategoryNode;
import nl.cloudfarming.client.fleet.model.Machine;
import nl.cloudfarming.client.util.swing.ASBodyPanel;
import org.openide.nodes.Node;
import org.openide.nodes.Node.Property;
import org.openide.util.ImageUtilities;
import org.openide.util.NbBundle.Messages;
import org.openide.windows.TopComponent;

/**
 * Top component which displays something.
 */
@TopComponent.Description(preferredID = "OverviewTopComponent",
iconBase = "nl/cloudfarming/client/fleet/machine/fleet.png",
persistenceType = TopComponent.PERSISTENCE_NEVER)
@Messages({
    "unknownPropertyValue=unknown"
})
public class OverviewTopComponent extends TopComponent implements PropertyChangeListener {

    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.fleet.machine");
    private static Map<Machine, OverviewTopComponent> overviewForMachine = new HashMap<>();
    private PropertyTableModel propertyListModel = new PropertyTableModel();
    protected Machine machine;
    private OverviewPanel overviewPanel;
    private ASBodyPanel overviewDecorator;
    private ImageIcon icon = ImageUtilities.loadImageIcon("nl/cloudfarming/client/fleet/machine/windows/fleet32.png", true);

    public OverviewTopComponent() {
        //calling this directly is not allowed
        SwingUtilities.invokeLater(new Runnable() {

            @Override
            public void run() {
                setToolTipText(Bundle.HINT_OverviewTopComponent());
            }
        });

    }

    private OverviewTopComponent(Machine machine) {
        this();
        this.machine = machine;
        
        setLayout(new BorderLayout());

        overviewPanel = new OverviewPanel();
        overviewPanel.setPropertyListModel(propertyListModel);

        overviewDecorator = new ASBodyPanel(overviewPanel);
        overviewDecorator.setIcon(icon);
        add(overviewDecorator);
    }

    public static OverviewTopComponent findInstance(Machine machine, MachineCategoryNode.MachineMetaInfo metaInfo) {
        OverviewTopComponent overview = overviewForMachine.get(machine);
        if (overview == null) {
            overview = new OverviewTopComponent(machine);
            overview.initTopComponentIcon(metaInfo.getSmallIconResource());
            overview.updateHeader(machine.getName(), metaInfo.getLargeIconResource());
            overviewForMachine.put(machine, overview);
        }
        return overview;
    }
    
    private void updateHeader(String name, String iconResource) {
        overviewDecorator.setTitle(name);
        if(iconResource != null) {
            overviewDecorator.setIcon(ImageUtilities.loadImageIcon(iconResource, true));
        }
    }
    
    private void initTopComponentIcon(String iconResource) {
        if(iconResource != null) {
            setIcon(ImageUtilities.loadImage(iconResource, true));
        }
    }

    private void updateOverview() {
        overviewDecorator.setTitle(machine.getName());
        setName(machine.getName());

        Node.PropertySet[] propertySets = machine.createSheet().toArray();
        for (Node.PropertySet propertySet : propertySets) {
            for (Property<?> property : propertySet.getProperties()) {
                String propertyName = property.getDisplayName();
                String propertyValue = getPropertyValue(property);
                propertyListModel.addRow(propertyName, propertyValue);
            }
        }
    }

    private String getPropertyValue(Property<?> property) {
        String propertyValue = Bundle.unknownPropertyValue();
        try {
            if (propertyHasValidValue(property)) {
                propertyValue = property.getValue().toString();
            }
        } catch (IllegalAccessException | InvocationTargetException ex) {
            LOGGER.log(Level.WARNING, "Could not access property value for machine", ex.getMessage());
        } finally {
            return propertyValue;
        }
    }

    private boolean propertyHasValidValue(Property<?> property) throws IllegalAccessException, InvocationTargetException {
        return property.getValue() != null && !property.getValue().toString().trim().isEmpty();
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 662, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 520, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables

    @Override
    public void componentOpened() {
        machine.addPropertyChangeListener(this);
        updateOverview();
    }

    @Override
    public void componentClosed() {
        machine.removePropertyChangeListener(this);
    }

    void writeProperties(java.util.Properties p) {
        // better to version settings since initial version as advocated at
        // http://wiki.apidesign.org/wiki/PropertyFiles
        p.setProperty("version", "1.0");
        // TODO store your settings
    }

    void readProperties(java.util.Properties p) {
        String version = p.getProperty("version");
        // TODO read your settings according to their version
    }

    @Override
    public void propertyChange(PropertyChangeEvent evt) {
        updateOverview();
    }

    private class PropertyTableModel extends DefaultTableModel {

        private String[] columns = {"Property", "Value"};

        public PropertyTableModel() {
            for (String columnName : columns) {
                addColumn(columnName);
            }
        }

        public void addRow(String propertyName, String propertyValue) {
            if (tableContainsPropertyWithName(propertyName)) {
                int rowPosition = getRowIndexForPropertyName(propertyName);
                setValueAt(propertyValue, rowPosition, 1);
            } else {
                String[] rowData = {propertyName, propertyValue};
                addRow(rowData);
            }
        }

        public int getRowIndexForPropertyName(String propertyName) {
            for (int i = 0; i < getRowCount(); i++) {
                if (getValueAt(i, 0).toString().equalsIgnoreCase(propertyName)) {
                    return i;
                }
            }

            return -1;
        }

        public boolean tableContainsPropertyWithName(String propertyName) {
            return getRowIndexForPropertyName(propertyName) != -1;
        }
    }
}
